@@ -1,5 +1,5 @@
 /* XQF - Quake server browser and launcher
- * Copyright (C) 1998,1999 Roman Pozlevich <roma@botik.ru>
+ * Copyright (C) 1998-2000 Roman Pozlevich <roma@botik.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,7 +34,9 @@
 #include <sys/param.h>	/* MAXHOSTNAMELEN */
 #include <time.h>	/* time */
 
+#ifndef h_errno
 extern int h_errno;	/* h_errno */
+#endif
 
 #include <gtk/gtk.h>
 
@@ -43,7 +45,6 @@ extern int h_errno;	/* h_errno */
 #include "host.h"
 #include "utils.h"
 
-
 #ifndef MAXHOSTNAMELEN
 # define MAXHOSTNAMELEN		64
 #endif
@@ -505,7 +506,8 @@ static void dns_resolve (char *str) {
 
   if (dns_workers_num >= DNS_MAX_CHILDREN || !dns_dispatch_to_worker (str)) {
 #ifdef DEBUG
-    fprintf (stderr, "DNS Master> \"%s\" is put to the waiting queue\n", str);
+    fprintf (stderr, "DNS Master> \"%s\" is put to the waiting queue\n",
+                     str);
 #endif
     dns_queue_add (strdup (str));
   }
@@ -652,7 +654,8 @@ static void dns_master_mainloop (void) {
 
 #ifdef DEBUG
     fprintf (stderr, "DNS Master> %d workers, queue is %s\n", 
-                            dns_workers_num, (q_head)? "not empty" : "empty");
+                            dns_workers_num,
+			    (q_head)? "not empty" : "empty");
 #endif
 
     FD_ZERO (&readfds);
