@@ -1,5 +1,5 @@
 /* XQF - Quake server browser and launcher
- * Copyright (C) 1998 Roman Pozlevich <roma@botik.ru>
+ * Copyright (C) 1998-2000 Roman Pozlevich <roma@botik.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,8 @@
 #include "utils.h"
 #include "flt-player.h"
 #include "filter.h"
+#include "debug.h"
+#include "gnuconfig.h"
 
 
 #define REGCOMP_FLAGS 	(REG_EXTENDED | REG_NOSUB | REG_ICASE)
@@ -63,19 +65,22 @@ enum {
 };
 
 static char *mode_symbols[3] = {
-  "string",
-  "substr",
-  "regexp"
+  N_("string"),
+  N_("substr"),
+  N_("regexp")
 };
 
 static const char *mode_names[3] = { 
-  "string", 
-  "substring",
-  "regular expression"
+  N_("string"), 
+  N_("substring"),
+  N_("regular expression")
 };
 
 
-int player_filter (struct server *s) {
+int player_filter (struct server *s, struct server_filter_vars *vars) {
+  /* The 'vars' is ignored in this function, however, since we need it
+     for applying the server filter, we will put it in the arguments. */
+
   GSList *list;
   GSList *plist;
   struct player_pattern *pp;
@@ -439,7 +444,7 @@ static void show_pattern_error (int row) {
   pp = (struct player_pattern *) list->data;
 
   if (pp->error) {
-    dialog_ok ("XQF: Error", "Regular Expression Error!\n\n%s\n\n%s.", 
+    dialog_ok (_("XQF: Error"), _("Regular Expression Error!\n\n%s\n\n%s."), 
                                                       pp->pattern, pp->error);
   }
 }
@@ -533,6 +538,8 @@ static void delete_pattern_callback (GtkWidget *widget, gpointer data) {
   struct player_pattern *pp;
   int row;
 
+  debug(5,"delete_pattern_callback(widget=%x,data=%x)",widget,data);
+
   if (current_row < 0)
     return;
 
@@ -555,6 +562,8 @@ static void delete_pattern_callback (GtkWidget *widget, gpointer data) {
 static void pattern_clist_adjust_visibility (int row, int direction) {
   GtkVisibility vis;
 
+  debug(5,"pattern_clist_adjust_visibility(row=%d,direction=%d)",row,direction);
+
   vis = gtk_clist_row_is_visible (GTK_CLIST (pattern_clist), row);
   if (vis != GTK_VISIBILITY_FULL) {
     gtk_clist_moveto (GTK_CLIST (pattern_clist), row, 0, 
@@ -564,33 +573,39 @@ static void pattern_clist_adjust_visibility (int row, int direction) {
 
 
 static void move_up_down_pattern_callback (GtkWidget *widget, int dir) {
-  GSList *link;
-  struct player_pattern *pp;
   int row = current_row;
 
+  debug(5,"move_up_down_pattern_callback(widget=%x, dir=%d) row=%d",widget,dir,row);
+
   if ((dir != -1 || row <= 0) &&
       (dir != 1  || row < 0 || row == g_slist_length (curplrs) - 1)) {
     return;
   }
-
+/*
   link = g_slist_nth (curplrs, row + dir);
   pp = (struct player_pattern *) link->data;
   curplrs = g_slist_remove_link (curplrs, link);
   curplrs = g_slist_insert (curplrs, pp, row);
-
+*/
+  debug(5,"gtk_clist_swap_rows(..., %d,%d)",row,row+dir);
   gtk_clist_swap_rows (GTK_CLIST (pattern_clist), row, row + dir);
-  current_row += dir;
+
+  // adjust current_row because pattern_clist_row_move_callback
+  // does not know about the direction
+  current_row = row+dir;
 
   pattern_clist_adjust_visibility (current_row, dir);
 }
 
 
 static void pattern_clist_row_move_callback (GtkWidget *widget, 
-                                                       int source, int dest) {
+                                                       int source, int dest, gpointer data) {
   GtkCList *clist = GTK_CLIST (widget);
   GSList *link;
   struct player_pattern *pp;
 
+  debug(5,"pattern_clist_row_move_callback(widget=%d,source=%d,dest=%d)",widget,source,dest);
+
   if (source < 0 || dest < 0 || source == dest || 
                                  source > clist->rows || dest > clist->rows) {
     return;
@@ -642,7 +657,7 @@ static GtkWidget *player_filter_pattern_editor (void) {
 
   /* Pattern Entry */
 
-  label = gtk_label_new ("Pattern");
+  label = gtk_label_new (_("Pattern"));
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, 0, 0, 0, 0);
   gtk_widget_show (label);
 
@@ -677,7 +692,7 @@ static GtkWidget *player_filter_pattern_editor (void) {
 
   /* Comment */
 
-  frame = gtk_frame_new ("Pattern Comment");
+  frame = gtk_frame_new (_("Pattern Comment"));
   gtk_box_pack_end (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
 
   hbox = gtk_hbox_new (FALSE, 0);
@@ -730,13 +745,13 @@ void player_filter_page (GtkWidget *notebook) {
   GtkWidget *pixmap;
   GtkWidget *button;
   GtkWidget *peditor;
-  char *titles[5] = { "", "", "", "Mode", "Pattern" };
+  char *titles[5] = { "", "", "", N_("Mode"), N_("Pattern") };
   int i;
 
   page_hbox = gtk_hbox_new (FALSE, 8);
   gtk_container_set_border_width (GTK_CONTAINER (page_hbox), 8);
 
-  label = gtk_label_new ("Player Filter");
+  label = gtk_label_new (_("Player Filter"));
   gtk_widget_show (label);
 
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page_hbox, label);
@@ -793,13 +808,13 @@ void player_filter_page (GtkWidget *notebook) {
   vbox = gtk_vbox_new (FALSE, 4);
   gtk_box_pack_start (GTK_BOX (page_hbox), vbox, FALSE, FALSE, 0);
 
-  button = gtk_button_new_with_label (" New ");
+  button = gtk_button_new_with_label (_("New"));
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
                                 GTK_SIGNAL_FUNC (new_pattern_callback), NULL);
   gtk_widget_show (button);
 
-  delete_button = gtk_button_new_with_label (" Delete ");
+  delete_button = gtk_button_new_with_label (_("Delete"));
   gtk_box_pack_start (GTK_BOX (vbox), delete_button, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (delete_button), "clicked",
                              GTK_SIGNAL_FUNC (delete_pattern_callback), NULL);
@@ -811,13 +826,13 @@ void player_filter_page (GtkWidget *notebook) {
   vbox2 = gtk_vbox_new (FALSE, 4);
   gtk_container_add (GTK_CONTAINER (alignment), vbox2);
 
-  up_button = gtk_button_new_with_label (" Up ");
+  up_button = gtk_button_new_with_label (_("Up"));
   gtk_box_pack_start (GTK_BOX (vbox2), up_button, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (up_button), "clicked",
                 GTK_SIGNAL_FUNC (move_up_down_pattern_callback), (void *) -1);
   gtk_widget_show (up_button);
 
-  down_button = gtk_button_new_with_label (" Down ");
+  down_button = gtk_button_new_with_label (_("Down"));
   gtk_box_pack_start (GTK_BOX (vbox2), down_button, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (down_button), "clicked",
                  GTK_SIGNAL_FUNC (move_up_down_pattern_callback), (void *) 1);
