@@ -1,5 +1,5 @@
 /* XQF - Quake server browser and launcher
- * Copyright (C) 1998,1999 Roman Pozlevich <roma@botik.ru>
+ * Copyright (C) 1998-2000 Roman Pozlevich <roma@botik.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@ struct game {
   void (*analyze_serverinfo) (struct server *s);
   int (*config_is_valid) (struct server *s);
   int (*write_config) (const struct condef *con);
-  void (*exec_client) (const struct condef *con, int forkit);
+  int (*exec_client) (const struct condef *con, int forkit);
   GList * (*custom_cfgs) (char *dir, char *game);
   void (*save_info) (FILE *f, struct server *s);
 
@@ -72,5 +72,4 @@ extern	enum server_type id2type (const char *id);
 extern	const char *type2id (enum server_type type);
 extern	GtkWidget *game_pixmap_with_label (enum server_type type);
 
-
 #endif /* __GAME_H__ */
