@@ -1,5 +1,5 @@
 /* XQF - Quake server browser and launcher
- * Copyright (C) 1998,1999 Roman Pozlevich <roma@botik.ru>
+ * Copyright (C) 1998-2000 Roman Pozlevich <roma@botik.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,18 +40,27 @@
 #include "xpm/q1.xpm"
 #include "xpm/q2.xpm"
 #include "xpm/q3.xpm"
+#include "xpm/wo.xpm"
 #include "xpm/hex.xpm"
+#include "xpm/hw.xpm"
 #include "xpm/sn.xpm"
 #include "xpm/hl.xpm"
 #include "xpm/kp.xpm"
+#include "xpm/sfs.xpm"
+#include "xpm/t2.xpm"
 #include "xpm/hr2.xpm"
 #include "xpm/un.xpm"
+#include "xpm/rune.xpm"
+#include "xpm/descent3.xpm"
+
+#include "xpm/gamespy3d.xpm"
 
 #include "xpm/green-plus.xpm"
 #include "xpm/red-minus.xpm"
 
 #include "xpm/man-black.xpm"
 #include "xpm/man-red.xpm"
+#include "xpm/man-yellow.xpm"
 
 #include "xpm/group-red.xpm"
 #include "xpm/group-green.xpm"
@@ -68,7 +77,7 @@
 #include "xpm/server-down.xpm"
 #include "xpm/server-to.xpm"
 #include "xpm/server-error.xpm"
-
+#include "xpm/locked.xpm"
 
 struct pixmap update_pix;
 struct pixmap refresh_pix;
@@ -79,6 +88,8 @@ struct pixmap connect_pix;
 struct pixmap observe_pix;
 struct pixmap record_pix;
 
+// Workaround for ppc compiler bug
+// which does not initialize arrays with 0
 struct pixmap filter_pix[2];
 struct pixmap filter_cfg_pix[2];
 
@@ -86,18 +97,26 @@ struct pixmap q_pix;
 struct pixmap q1_pix;
 struct pixmap q2_pix;
 struct pixmap q3_pix;
+struct pixmap wo_pix;
 struct pixmap hex_pix;
+struct pixmap hw_pix;
 struct pixmap sn_pix;
 struct pixmap hl_pix;
 struct pixmap kp_pix;
+struct pixmap sfs_pix;
+struct pixmap t2_pix;
 struct pixmap hr_pix;
 struct pixmap un_pix;
+struct pixmap rune_pix;
+struct pixmap descent3_pix;
+struct pixmap gamespy3d_pix;
 
 struct pixmap gplus_pix;
 struct pixmap rminus_pix;
 
 struct pixmap man_black_pix;
 struct pixmap man_red_pix;
+struct pixmap man_yellow_pix;
 
 struct pixmap group_pix[3];
 struct pixmap buddy_pix[9];
@@ -105,6 +124,7 @@ struct pixmap buddy_pix[9];
 struct pixmap error_pix;
 
 struct pixmap server_status[5];
+struct pixmap locked_pix;
 
 static GdkGC *pixmaps_gc;
 static GdkGC *masks_gc;
@@ -179,18 +199,26 @@ void free_pixmaps (void) {
   free_pixmap (&q1_pix);
   free_pixmap (&q2_pix);
   free_pixmap (&q3_pix);
+  free_pixmap (&wo_pix);
   free_pixmap (&hex_pix);
+  free_pixmap (&hw_pix);
   free_pixmap (&sn_pix);
   free_pixmap (&hl_pix);
   free_pixmap (&kp_pix);
+  free_pixmap (&sfs_pix);
+  free_pixmap (&t2_pix);
   free_pixmap (&hr_pix);
   free_pixmap (&un_pix);
+  free_pixmap (&rune_pix);
+  free_pixmap (&descent3_pix);
+  free_pixmap (&gamespy3d_pix);
 
   free_pixmap (&gplus_pix);
   free_pixmap (&rminus_pix);
 
   free_pixmap (&man_black_pix);
   free_pixmap (&man_red_pix);
+  free_pixmap (&man_yellow_pix);
 
   for (i = 0; i < 3; i++)
     free_pixmap (&group_pix[i]);
@@ -203,6 +231,8 @@ void free_pixmaps (void) {
   for (i = 0; i < 5; i++)
     free_pixmap (&server_status[i]);
 
+  free_pixmap (&locked_pix);
+
   if (pixmaps_gc) {
     gdk_gc_destroy (pixmaps_gc);
     pixmaps_gc = NULL;
@@ -240,18 +270,26 @@ void init_pixmaps (GtkWidget *window) {
   create_pixmap (window, &q1_pix, q1_xpm);
   create_pixmap (window, &q2_pix, q2_xpm);
   create_pixmap (window, &q3_pix, q3_xpm);
+  create_pixmap (window, &wo_pix, wo_xpm);
   create_pixmap (window, &hex_pix, hex_xpm);
+  create_pixmap (window, &hw_pix, hw_xpm);
   create_pixmap (window, &sn_pix, sn_xpm);
   create_pixmap (window, &hl_pix, hl_xpm);
   create_pixmap (window, &kp_pix, kp_xpm);
+  create_pixmap (window, &sfs_pix, sfs_xpm);
+  create_pixmap (window, &t2_pix, t2_xpm);
   create_pixmap (window, &hr_pix, hr2_xpm);
   create_pixmap (window, &un_pix, un_xpm);
+  create_pixmap (window, &rune_pix, rune_xpm);
+  create_pixmap (window, &descent3_pix, descent3_xpm);
+  create_pixmap (window, &gamespy3d_pix, gamespy3d_xpm);
 
   create_pixmap (window, &gplus_pix, green_plus_xpm);
   create_pixmap (window, &rminus_pix, red_minus_xpm);
 
   create_pixmap (window, &man_black_pix, man_black_xpm);
   create_pixmap (window, &man_red_pix, man_red_xpm);
+  create_pixmap (window, &man_yellow_pix, man_yellow_xpm);
 
   create_pixmap (window, &group_pix[0], group_red_xpm);
   create_pixmap (window, &group_pix[1], group_green_xpm);
@@ -268,6 +306,7 @@ void init_pixmaps (GtkWidget *window) {
   create_pixmap (window, &server_status[4], server_error_xpm);
 
   create_pixmap (window, &error_pix, error_xpm);
+  create_pixmap (window, &locked_pix, locked_xpm);
 }
 
 
