@@ -1,5 +1,5 @@
 /* XQF - Quake server browser and launcher
- * Copyright (C) 1998 Roman Pozlevich <roma@botik.ru>
+ * Copyright (C) 1998-2000 Roman Pozlevich <roma@botik.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,39 +32,53 @@
 #endif
 
 enum {
-  PREF_PAGE_PLAYER = 0,
+  PREF_PAGE_GENERAL = 0,
   PREF_PAGE_GAMES,
   PREF_PAGE_APPEARANCE,
   PREF_PAGE_QSTAT,
-  PREF_PAGE_QWQ2
 };
 
 extern	char *user_rcdir;
 
-extern	int	show_hostnames;
-extern	int	show_default_port;
-
-extern  int 	default_rate;
-extern  int 	default_cl_nodelta;
-extern  int 	default_cl_predict;
-extern  int 	default_noaim;
-extern  int 	default_pushlatency;
-extern  int 	default_nosound;
-extern  int 	default_nocdaudio;
-extern  char 	*default_name;
-extern  char 	*default_team;
+extern  char 	*default_q1_name;
+extern  char 	*default_qw_name;
+extern  char 	*default_q2_name;
+extern  char 	*default_t2_name;
+extern  char 	*default_qw_team;
 extern  char 	*default_qw_skin;
 extern  char 	*default_q2_skin;
 extern  int 	default_q1_top_color;
 extern  int 	default_q1_bottom_color;
 extern  int 	default_qw_top_color;
 extern  int 	default_qw_bottom_color;
-extern  int 	default_noskins;
+
+extern  int 	default_qw_rate;
+extern  int 	default_q2_rate;
+extern  int 	default_qw_cl_nodelta;
+extern  int 	default_q2_cl_nodelta;
+extern  int 	default_qw_cl_predict;
+extern  int 	default_q2_cl_predict;
+extern  int 	default_noaim;
+extern  int 	default_qw_noskins;
+extern  int 	default_q2_noskins;
+extern  int 	default_b_switch;
+extern  int 	default_w_switch;
+
+extern	int 	pushlatency_mode;
+extern	int 	pushlatency_value;
+
+extern  int	default_qw_is_quakeforge;
+
+extern  int 	default_nosound;
+extern  int 	default_nocdaudio;
+
+extern	int	show_hostnames;
+extern	int	show_default_port;
 
 extern	int	default_terminate;
 extern	int	default_iconify;
-extern  int 	default_b_switch;
-extern  int 	default_w_switch;
+extern	int	default_launchinfo;
+extern	int	default_prelaunchexec;
 extern  int 	default_save_lists;
 extern	int 	default_save_srvinfo;
 extern	int 	default_save_plrinfo;
@@ -78,6 +92,23 @@ extern	int	default_refresh_on_update;
 extern	int	maxretries;
 extern	int	maxsimultaneous;
 
+/* Quake 3 settings */
+struct q3engineopts
+{
+  char *masterprotocol;
+  struct game *game;
+  int vmfix;
+  int rafix;
+  int setfs_game;
+};
+
+// Quake 3 Arena
+extern struct q3engineopts q3_opts;
+// Return to Castle Wolfenstein
+extern struct q3engineopts wo_opts;
+// Any other game with Q3 engine
+extern struct q3engineopts generic_q3_opts;
+
 extern	int set_working_dir (char *dir);
 extern  void preferences_dialog (int page_num);
 extern	int fix_qw_player_color (int color);
