@@ -1,5 +1,5 @@
-/* XQF - Quake server browser and launcher
- * Copyright (C) 1998,1999 Roman Pozlevich <roma@botik.ru>
+/* XQF - Quake server browser and launcher Copyright (C) 1998-2000 Roman
+ * Pozlevich <roma@botik.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,9 @@
 #include <arpa/inet.h>	/* inet_aton, inet_ntoa */
 #include <errno.h>	/* errno */
 
+#ifndef errno
 extern	int errno;
+#endif
 
 #include <gtk/gtk.h>
 
@@ -45,6 +47,13 @@ extern	int errno;
 #include "zipped.h"
 #include "stat.h"
 
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#define _(string) gettext(string)
+#else
+#define _(string) (string)
+#endif
+
 
 struct master *favorites = NULL;
 GSList *master_groups = NULL;
@@ -58,6 +67,7 @@ static void save_list (FILE *f, struct master *m) {
 
   if (!m->servers)
     return;
+  debug (6, "save_list() -- Saving Server List \"%s\"", m->name );
 
 #ifdef DEBUG
   fprintf (stderr, "Saving server list \"%s\"...\n", m->name);
@@ -68,18 +78,25 @@ static void save_list (FILE *f, struct master *m) {
   }
   else {
     if (m->url) {
-      fprintf (f, "[%s]\n", m->url);
+      fprintf (f, "[%s %s]\n", games[m->type].id, m->url);
     }
     else {
-      fprintf (f, "[%s%s:%d]\n", PREFIX_MASTER,
-               (m->hostname)? m->hostname : inet_ntoa (m->host->ip), m->port);
+      if (m->master_type == 1)
+	{
+	fprintf (f, "[%s %s%s:%d]\n", games[m->type].id, PREFIX_GMASTER,
+        	(m->hostname)? m->hostname : inet_ntoa (m->host->ip), m->port);
+        }
+      else
+        {
+      	fprintf (f, "[%s %s%s:%d]\n", games[m->type].id, PREFIX_MASTER,
+		(m->hostname)? m->hostname : inet_ntoa (m->host->ip), m->port);
+	}
     }
   }
 
   for (srv = m->servers; srv; srv = srv->next) {
     s = (struct server *) srv->data;
-    fprintf (f, "%s %s:%d\n", games[s->type].id, 
-                                            inet_ntoa (s->host->ip), s->port);
+    fprintf (f, "%s %s:%d\n", games[s->type].id, inet_ntoa (s->host->ip), s->port);
   }
 
   fprintf (f, "\n");
@@ -90,14 +107,15 @@ void save_favorites (void) {
   FILE *f;
   char *realname;
 
+  debug (6, "save_favorites() --");
   realname = file_in_dir (user_rcdir, FILENAME_FAVORITES);
 
   while (1) {
     f = fopen (realname, "w");
 
     if (!f) {
-      if (dialog_yesno (NULL, 0, "Retry", "Skip", 
-			"Cannot write to file %s\nSystem Error: %s\n", 
+      if (dialog_yesno (NULL, 0, _("Retry"), _("Skip"), 
+			_("Cannot write to file %s\nSystem Error: %s\n"), 
 			realname, g_strerror (errno))) {
 	continue;
       }
@@ -118,6 +136,7 @@ static void save_lists (GSList *list, const char *filename) {
   char *realname;
   struct zstream z;
 
+  debug (6, "save_lists() -- %s", filename);
   realname = file_in_dir (user_rcdir, filename);
 
   if (!list) {
@@ -171,7 +190,7 @@ static void save_server_info (const char *filename, GSList *servers) {
 }
 
 
-static struct master *find_master_server (char *addr, unsigned short port) {
+static struct master *find_master_server (char *addr, unsigned short port, char *str) {
   GSList *list;
   struct master *m;
   struct in_addr ip;
@@ -181,15 +200,25 @@ static struct master *find_master_server (char *addr, unsigned short port) {
 
   for (list = all_masters; list; list = list->next) {
     m = (struct master *) list->data;
-    if (m->port && m->port == port) {
-      if (m->hostname) {
+    if (m->port && m->port == port) 
+    {
+      if (m->hostname) 
+      {
 	if (!g_strcasecmp (m->hostname, addr))
-	  return m;
+	{
+          if (!str) // pre 0.9.4e list file
+            return m;
+	  if (!g_strcasecmp (games[m->type].id, str)) // list file in new format
+	    return m;
+	}
       }
-      else {
-	if (m->host && inet_aton (addr, &ip) && 
-	                                    ip.s_addr == m->host->ip.s_addr) {
-	  return m;
+      else 
+      {
+	if (m->host && inet_aton (addr, &ip) && ip.s_addr == m->host->ip.s_addr) {
+          if (!str) // pre 0.9.4e list file
+            return m;
+	  if (!g_strcasecmp (games[m->type].id, str))
+  	    return m;
 	}
       }
     }
@@ -298,7 +327,7 @@ static struct master *find_master_url (char *url) {
 }
 
 
-static struct master *read_list_parse_master (char *str) {
+static struct master *read_list_parse_master (char *str, char *str2) {
   char *addr;
   unsigned short port;
   struct master *m;
@@ -308,7 +337,15 @@ static struct master *read_list_parse_master (char *str) {
 
   if (g_strncasecmp (str, PREFIX_MASTER, sizeof (PREFIX_MASTER) - 1) == 0) {
     if (parse_address (str + sizeof (PREFIX_MASTER) - 1, &addr, &port)) {
-      m = find_master_server (addr, port);
+      m = find_master_server (addr, port, str2);
+      g_free (addr);
+      return m;
+    }
+  }
+
+  if (g_strncasecmp (str, PREFIX_GMASTER, sizeof (PREFIX_GMASTER) - 1) == 0) {
+    if (parse_address (str + sizeof (PREFIX_GMASTER) - 1, &addr, &port)) {
+      m = find_master_server (addr, port, str2);
       g_free (addr);
       return m;
     }
@@ -331,12 +368,20 @@ static void master_add_server (struct master *m, char *str,
   struct server *s;
   struct userver *us;
 
+  debug (6, "master_add_server() -- Master %lx", m);
   if (parse_address (str, &addr, &port)) {
     h = host_add (addr);
     if (h) {						/* IP address */
       host_ref (h);
-      if ((s = server_add (h, port, type)) != NULL)
+      if ((s = server_add (h, port, type)) != NULL) {
 	m->servers = server_list_prepend (m->servers, s);
+	/* Since the server_add increments the ref count, and 
+	   server_list_prepend ups the ref_count, we should
+	   unref it once because we are only keeping it in
+	   one list after this function.
+	*/
+	server_unref (s);
+      }
       host_unref (h);
     }
     else {						/* hostname */
@@ -372,16 +417,28 @@ static void read_lists (const char *filename) {
     if (n < 1)
       continue;
 
-    if (token[0][0] == '[') {
-      ch = strchr (token[0] + 1, ']');
+    if (token[0][0] == '[') {  // line is a master
+      ch = strchr (token[0] + 1, ']'); // does token 0 have a ]?
+      if (ch) {	// it's a favorites or pre 0.9.4e lists file
+       *ch = '\0';
+
+      if (m && m->servers)
+        m->servers = g_slist_reverse (m->servers);
+
+      m = read_list_parse_master (token[0] + 1, NULL);
+    }
+    else {
+      ch = strchr (token[1], ']'); // does token 1 have a ]?
       if (ch) {
-	*ch = '\0';
 
-	if (m && m->servers)
-	  m->servers = g_slist_reverse (m->servers);
+      *ch = '\0';
 
-	m = read_list_parse_master (token[0] + 1);
+      if (m && m->servers)
+      m->servers = g_slist_reverse (m->servers);
+
+      m = read_list_parse_master (token[1], token[0] + 1); // master, type
       }
+     }
     }
     else {
       if (!m || n < 2)
@@ -554,7 +611,7 @@ struct master *add_master (char *path, char *name, enum server_type type,
 	}
       }
 
-      m = find_master_server (addr, port);
+      m = find_master_server (addr, port, games[type].id);
 
       if (lookup_only) {
 	g_free (addr);
@@ -578,8 +635,12 @@ struct master *add_master (char *path, char *name, enum server_type type,
 	g_free (addr);
 	return m;
       }
-      else {
+      else
+       {
 	m = create_master (name, type, FALSE);
+
+        m->master_type = 0; // Regular master
+
 	h = host_add (addr);
 	if (h) {
 	  m->host = h;
@@ -594,6 +655,65 @@ struct master *add_master (char *path, char *name, enum server_type type,
 
     }
   }
+
+  else if (g_strncasecmp (path, PREFIX_GMASTER, sizeof (PREFIX_GMASTER) - 1) == 0) {
+    if (parse_address (path + sizeof (PREFIX_GMASTER) - 1, &addr, &port)) {
+
+      if (!port) {
+	if (games[type].default_master_port) {
+	  port = games[type].default_master_port;
+	}
+	else {
+	  g_free (addr);
+	  return NULL;
+	}
+      }
+
+      m = find_master_server (addr, port, games[type].id);
+
+      if (lookup_only) {
+	g_free (addr);
+	return m;
+      }
+
+      if (m) {
+	if (user) {
+	  /* Master renaming is forced by user */
+	  g_free (m->name);
+	  m->name = g_strdup (name);
+	  m->user = TRUE;
+	}
+	else {
+	  /* Automatically rename masters that are not edited by user */
+	  if (!m->user) {
+	    g_free (m->name);
+	    m->name = g_strdup (name);
+	  }
+	}
+	g_free (addr);
+	return m;
+      }
+      else 
+      {
+	m = create_master (name, type, FALSE);
+
+        m->master_type = 1; // Gamespy master
+
+	h = host_add (addr);
+	if (h) {
+	  m->host = h;
+	  host_ref (h);
+	  g_free (addr);
+	}
+	else {
+	  m->hostname = addr;
+	}
+	m->port = port;
+      }
+
+    }
+  }
+
   else {
     if (g_strncasecmp (path, PREFIX_URL_HTTP, sizeof (PREFIX_URL_HTTP) - 1) 
                                                                        == 0) {
@@ -700,6 +820,7 @@ static char *builtin_masters_update_info[] = {
   "ADD Q3S master://master3.idsoftware.com id",
   "ADD Q3S master://q3master.splatterworld.de Germany",
   "ADD Q3S master://q3.golsyd.net.au Australia",
+  "ADD Q3S master://q3master.barrysworld.com:27950 BarrysWorld",
 #endif
 
   "ADD SNS http://www.gameaholic.com/servers/qspy-sin Gameaholic.Com",
@@ -711,9 +832,34 @@ static char *builtin_masters_update_info[] = {
 #endif
 
   "ADD Q2S:KP http://www.gameaholic.com/servers/qspy-kingpin Gameaholic.Com",
+  "ADD Q2S:KP http://www.ogn.org:6666 OGN",
 
   "ADD Q2S:HR http://www.gameaholic.com/servers/qspy-heretic2 Gameaholic.Com",
 
+#ifdef QSTAT_HAS_UNREAL_SUPPORT
+  "ADD UNS gmaster://unreal.epicgames.com:28900 Epic",
+  "ADD UNS gmaster://utmaster.barrysworld.com:28909 BarrysWorld",
+#endif
+
+  "ADD T2S master://198.74.33.29:28002 NA West1",
+  "ADD T2S master://64.94.105.141:27999 NA West2",
+  "ADD T2S master://64.94.105.145:28000 NA West3",
+  "ADD T2S master://209.67.28.144:28002 NA East1",
+  "ADD T2S master://209.67.28..145:27999 NA East2",
+
+  "ADD WOS master://wolf.idsoftware.com:27950 id",
+
+  "ADD D3P master://gt.pxo.net:3445 PXO",
+  "ADD D3P http://www.gameaholic.com/servers/qspy-descent3 gameaholic.com",
+
+  "ADD SFS http://www.gameaholic.com/servers/qspy-soldieroffortune gameaholic.com",
+  "ADD GPS http://www.gameaholic.com/servers/qspy-noonelivesforever Noone Lives Forevere - gameaholic.com",
+  "ADD GPS http://www.gameaholic.com/servers/qspy-rallymasters Rally Masters - gameaholic.com",
+  "ADD GPS http://www.gameaholic.com/servers/qspy-turok2 Turok 2 - gameaholic.com",
+  "ADD GPS http://www.gameaholic.com/servers/qspy-shogo Shogo - gameaholic.com",
+
+  "ADD RUNESRV http://www.gameaholic.com/servers/qspy-rune gameaholic.com",
+
   NULL
 };
 
@@ -761,7 +907,14 @@ static void update_master_list_action (const char *action) {
 }
 
 
-static void update_master_list_builtin () {
+void update_master_list_builtin (void) {
+  char **ptr;
+
+  for (ptr = builtin_masters_update_info; *ptr; ptr++)
+    update_master_list_action (*ptr);
+}
+
+void update_master_list_web (void) {
   char **ptr;
 
   for (ptr = builtin_masters_update_info; *ptr; ptr++)
@@ -855,8 +1008,14 @@ static void save_master_list (void) {
       confstr = g_strjoin (" ", typeid, m->url, m->name, NULL);
     }
     else {
-      addr = g_strdup_printf (PREFIX_MASTER "%s:%d", 
-               (m->hostname)? m->hostname : inet_ntoa (m->host->ip), m->port);
+
+      if (m->master_type == 1)
+	      addr = g_strdup_printf (PREFIX_GMASTER "%s:%d", 
+        	       (m->hostname)? m->hostname : inet_ntoa (m->host->ip), m->port);
+      else
+      	      addr = g_strdup_printf (PREFIX_MASTER "%s:%d", 
+        	       (m->hostname)? m->hostname : inet_ntoa (m->host->ip), m->port);
+
       confstr = g_strjoin (" ", typeid, addr, m->name, NULL);
       g_free (addr);
     }
@@ -883,8 +1042,10 @@ void init_masters (int update) {
 
   load_master_list ();
 
-  if (update)
+  if (update) {
     update_master_list_builtin ();
+    config_sync ();
+  }
 
   compat_convert_favorites ();
 
@@ -898,6 +1059,7 @@ void init_masters (int update) {
 void free_masters (void) {
   GSList *tmp;
 
+  debug (6, "free_masters() --");
   save_favorites ();
 
   if (default_save_lists) {
@@ -907,7 +1069,7 @@ void free_masters (void) {
     g_slist_free (tmp);
 
     if (default_save_srvinfo) {
-      tmp = all_servers ();
+      tmp = all_servers (); /* free done in two lines */
       save_server_info (FILENAME_SRVINFO, tmp);
       server_list_free (tmp);
     }
@@ -941,6 +1103,7 @@ static void master_list_add (GSList **masters, GSList **servers,
                                         GSList **uservers, struct master *m) {
   GSList *tmp;
 
+  debug (6, "master_list_add() -- master '%s'", m->name);
   if (m->isgroup || m == favorites) {
     if (servers) {
       *servers = server_list_append_list (*servers, favorites->servers, 
@@ -989,7 +1152,7 @@ void master_selection_to_lists (GSList *list, GSList **masters,
                                         GSList **servers, GSList **uservers) {
   struct master *m;
   GSList *tmp;
-
+  debug (6, "master_selection_to_lists() --");
   for (tmp = list; tmp; tmp = tmp->next) {
     m = (struct master *) tmp->data;
     uservers_to_servers (&m->uservers, &m->servers);
