@@ -1,5 +1,5 @@
 /* XQF - Quake server browser and launcher
- * Copyright (C) 1998,1999 Roman Pozlevich <roma@botik.ru>
+ * Copyright (C) 1998-2000 Roman Pozlevich <roma@botik.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #define	FILENAME_SRVINFO	"srvinfo"
 
 #define	PREFIX_MASTER		"master://"
+#define	PREFIX_GMASTER		"gmaster://"
 #define	PREFIX_URL_HTTP		"http://"
 
 #define	ACTION_ADD		"ADD"
@@ -47,6 +48,8 @@ extern	void free_master (struct master *m);
 extern	void save_favorites (void);
 
 extern	void init_masters (int autoupdate);
+extern	void update_master_list_builtin (void);
+
 extern	void free_masters (void);
 
 extern	void collate_server_lists (GSList *masters, GSList **servers, 
