@@ -1,5 +1,5 @@
 /* XQF - Quake server browser and launcher
- * Copyright (C) 1998 Roman Pozlevich <roma@botik.ru>
+ * Copyright (C) 1998-2000 Roman Pozlevich <roma@botik.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,6 +40,13 @@
 #include "dialogs.h"
 #include "srv-prop.h"
 
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#define _(string) gettext(string)
+#else
+#define _(string) (string)
+#endif
+
 
 static  GtkWidget *password_entry;
 static  GtkWidget *spectator_entry;
@@ -71,23 +78,29 @@ void props_free_all (void) {
 }
 
 
-struct server_props *properties (struct server *s) {
-  struct server_props *p;
+static struct server_props *__properties (const struct host *h, 
+					  const unsigned short p) {
+  struct server_props *res;
   GSList *tmp;
 
-  if (!s)
+  if (!h || p == 0 || p > 65535)
     return NULL;
 
   for (tmp = props_list; tmp; tmp = tmp->next) {
-    p = (struct server_props *) tmp->data;
-    if (p->host == s->host && p->port == s->port)
-      return p;
+    res = (struct server_props *) tmp->data;
+    if (res->host == h && res->port == p)
+      return res;
   }
 
   return NULL;
 }
 
 
+struct server_props *properties (struct server *s) {
+  return __properties (s->host, s->port);
+}
+
+
 struct server_props *properties_new (struct host *host, unsigned short port) {
   struct server_props *p;
 
@@ -200,7 +213,9 @@ void props_load (void) {
       if (!h)
 	continue;
 
-      p = properties_new (h, port);
+      p = __properties (h, port);
+      if (!p)
+	p = properties_new (h, port);
     }
     else {
       if (!p)
@@ -212,14 +227,22 @@ void props_load (void) {
 
       *ptr++ = '\0';
 
-      if (strcmp (buf, "custom_cfg") == 0)
+      if (strcmp (buf, "custom_cfg") == 0) {
+	if (p->custom_cfg) g_free (p->custom_cfg);
 	p->custom_cfg = strdup_strip (ptr);
-      else if (strcmp (buf, "password") == 0)
+      }
+      else if (strcmp (buf, "password") == 0) {
+	if (p->server_password) g_free (p->server_password);
 	p->server_password = strdup_strip (ptr);
-      else if (strcmp (buf, "spectator_password") == 0)
+      }
+      else if (strcmp (buf, "spectator_password") == 0) {
+	if (p->spectator_password) g_free (p->spectator_password);
 	p->spectator_password = strdup_strip (ptr);
-      else if (strcmp (buf, "rcon_password") == 0)
+      }
+      else if (strcmp (buf, "rcon_password") == 0) {
+	if (p->rcon_password) g_free (p->rcon_password);
 	p->rcon_password = strdup_strip (ptr);
+      }
     }
   }
 
@@ -306,7 +329,7 @@ static GtkWidget *server_info_page (struct server *s) {
 
   gtk_table_set_col_spacing (GTK_TABLE (table), 1, 16);
 
-  label = gtk_label_new ("IP Address:");
+  label = gtk_label_new (_("IP Address:"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
   gtk_widget_show (label);
@@ -316,7 +339,7 @@ static GtkWidget *server_info_page (struct server *s) {
   gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 0, 1);
   gtk_widget_show (label);
 
-  label = gtk_label_new ("Port:");
+  label = gtk_label_new (_("Port:"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_table_attach_defaults (GTK_TABLE (table), label, 2, 3, 0, 1);
   gtk_widget_show (label);
@@ -328,7 +351,7 @@ static GtkWidget *server_info_page (struct server *s) {
   gtk_table_attach_defaults (GTK_TABLE (table), label, 3, 4, 0, 1);
   gtk_widget_show (label);
 
-  label = gtk_label_new ("Host Name:");
+  label = gtk_label_new (_("Host Name:"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 1, 2);
   gtk_widget_show (label);
@@ -340,7 +363,7 @@ static GtkWidget *server_info_page (struct server *s) {
     gtk_widget_show (label);
   }
 
-  label = gtk_label_new ("Refreshed:");
+  label = gtk_label_new (_("Refreshed:"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
   gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 2, 3);
   gtk_widget_show (label);
@@ -362,7 +385,7 @@ static GtkWidget *server_info_page (struct server *s) {
 
   /* Sources */
 
-  frame = gtk_frame_new ("Sources");
+  frame = gtk_frame_new (_("Sources"));
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
   gtk_box_pack_start (GTK_BOX (page_vbox), frame, FALSE, FALSE, 0);
 
@@ -414,7 +437,7 @@ static GtkWidget *server_info_page (struct server *s) {
   gtk_box_pack_end (GTK_BOX (hbox), customcfg_combo, FALSE, FALSE, 0);
   gtk_widget_show (customcfg_combo);
 
-  label = gtk_label_new ("Custom CFG:");
+  label = gtk_label_new (_("Custom CFG:"));
   gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
 
@@ -463,21 +486,21 @@ static GtkWidget *server_passwords_page (struct server *s) {
   gtk_table_set_col_spacings (GTK_TABLE (table), 8);
   gtk_box_pack_start (GTK_BOX (page_vbox), table, FALSE, FALSE, 0);
 
-  password_entry = passwd_entry ("Server Password",
+  password_entry = passwd_entry (_("Server Password"),
 				 (props)? props->server_password : NULL,
 				 table, 0);
 
   if ((games[s->type].flags & GAME_PASSWORD) == 0)
     gtk_widget_set_sensitive (password_entry, FALSE);
 
-  spectator_entry = passwd_entry ("Spectator Password",
+  spectator_entry = passwd_entry (_("Spectator Password"),
 				  (props)? props->spectator_password : NULL,
 				  table, 1);
 
   if ((games[s->type].flags & GAME_SPECTATE) == 0)
     gtk_widget_set_sensitive (spectator_entry, FALSE);
 
-  rcon_entry = passwd_entry ("RCon Password",
+  rcon_entry = passwd_entry (_("RCon Password"),
 			     (props)? props->rcon_password : NULL,
 			     table, 2);
 
@@ -504,7 +527,7 @@ void properties_dialog (struct server *s) {
   GtkWidget *label;
   char buf[256];
 
-  window = dialog_create_modal_transient_window ("Properties", 
+  window = dialog_create_modal_transient_window (_("Properties"), 
                                                            TRUE, FALSE, NULL);
   main_vbox = gtk_vbox_new (FALSE, 8);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 8);
@@ -549,12 +572,12 @@ void properties_dialog (struct server *s) {
   gtk_box_pack_start (GTK_BOX (main_vbox), notebook, FALSE, FALSE, 0);
 
   page = server_info_page (s);
-  label = gtk_label_new (" Info ");
+  label = gtk_label_new (_("Info"));
   gtk_widget_show (label);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
 
   page = server_passwords_page (s);
-  label = gtk_label_new (" Passwords ");
+  label = gtk_label_new (_("Passwords"));
   gtk_widget_show (label);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
 
@@ -569,7 +592,7 @@ void properties_dialog (struct server *s) {
   hbox = gtk_hbox_new (FALSE, 8);
   gtk_box_pack_start (GTK_BOX (main_vbox), hbox, FALSE, FALSE, 0);
 
-  button = gtk_button_new_with_label ("Cancel");
+  button = gtk_button_new_with_label (_("Cancel"));
   gtk_widget_set_usize (button, 80, -1);
   gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
@@ -577,7 +600,7 @@ void properties_dialog (struct server *s) {
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_widget_show (button);
 
-  button = gtk_button_new_with_label ("OK");
+  button = gtk_button_new_with_label (_("OK"));
   gtk_widget_set_usize (button, 80, -1);
   gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
