@@ -1,5 +1,5 @@
 /* XQF - Quake server browser and launcher
- * Copyright (C) 1998,1999 Roman Pozlevich <roma@botik.ru>
+ * Copyright (C) 1998-2000 Roman Pozlevich <roma@botik.ru>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include "game.h"
 #include "config.h"
 #include "sort.h"
+#include "gnuconfig.h"
 
 
 static GSList *xqf_windows = NULL;
@@ -38,33 +39,39 @@ GtkWidget *pane1_widget;
 GtkWidget *pane2_widget;
 GtkWidget *pane3_widget;
 
-static struct clist_column server_columns[7] = {
-  { "Name",    180,  GTK_JUSTIFY_LEFT,   NULL },
-  { "Address", 140,  GTK_JUSTIFY_LEFT,   NULL },
-  { "Ping",     45,  GTK_JUSTIFY_RIGHT,  NULL },
-  { "TO",       35,  GTK_JUSTIFY_RIGHT,  NULL },
-  { "Players",  65,  GTK_JUSTIFY_RIGHT,  NULL },
-  { "Map",      55,  GTK_JUSTIFY_LEFT,   NULL },
-  { "Game",     55,  GTK_JUSTIFY_LEFT,   NULL }
+/* If you add a column here to appear in the server
+   list, you need to also add an entry in sort.h and sort.c 
+*/
+static struct clist_column server_columns[10] = {
+  { N_("Name"),    180,  GTK_JUSTIFY_LEFT,   NULL },
+  { N_("Address"), 140,  GTK_JUSTIFY_LEFT,   NULL },
+  { N_("Ping"),     45,  GTK_JUSTIFY_RIGHT,  NULL },
+  { N_("TO"),       35,  GTK_JUSTIFY_RIGHT,  NULL },
+  { N_("Priv"),     20,  GTK_JUSTIFY_RIGHT,  NULL },
+  { N_("Players"),  65,  GTK_JUSTIFY_RIGHT,  NULL },
+  { N_("Map"),      55,  GTK_JUSTIFY_LEFT,   NULL },
+  { N_("Game"),     55,  GTK_JUSTIFY_LEFT,   NULL },
+  { N_("GameType"), 55,  GTK_JUSTIFY_LEFT,   NULL },
+  { N_("OS"),       35,  GTK_JUSTIFY_LEFT,   NULL }
 };
 
 struct clist_def server_clist_def = {
   CWIDGET_CLIST,
   "Server List",
   server_columns,
-  7,
+  9,
   GTK_SELECTION_EXTENDED,
   630, 270,
   SORT_SERVER_PING, GTK_SORT_ASCENDING
 };
 	
 static struct clist_column player_columns[6] = {
-  { "Name",    100,  GTK_JUSTIFY_LEFT,   NULL },
-  { "Frags",    50,  GTK_JUSTIFY_RIGHT,  NULL },
-  { "Colors",   60,  GTK_JUSTIFY_LEFT,   NULL },
-  { "Skin",     50,  GTK_JUSTIFY_LEFT,   NULL },
-  { "Ping",     45,  GTK_JUSTIFY_RIGHT,  NULL },
-  { "Time",     45,  GTK_JUSTIFY_LEFT,   NULL }
+  { N_("Name"),    100,  GTK_JUSTIFY_LEFT,   NULL },
+  { N_("Frags"),    50,  GTK_JUSTIFY_RIGHT,  NULL },
+  { N_("Colors"),   60,  GTK_JUSTIFY_LEFT,   NULL },
+  { N_("Skin"),     50,  GTK_JUSTIFY_LEFT,   NULL },
+  { N_("Ping"),     45,  GTK_JUSTIFY_RIGHT,  NULL },
+  { N_("Time"),     45,  GTK_JUSTIFY_LEFT,   NULL }
 };
 
 struct clist_def player_clist_def = {
@@ -78,8 +85,8 @@ struct clist_def player_clist_def = {
 };
 						  
 static struct clist_column srvinf_columns[2] = {
-  { "Rule",     90,  GTK_JUSTIFY_LEFT,   NULL },
-  { "Value",    80,  GTK_JUSTIFY_LEFT,   NULL }
+  { N_("Rule"),     90,  GTK_JUSTIFY_LEFT,   NULL },
+  { N_("Value"),    80,  GTK_JUSTIFY_LEFT,   NULL }
 };
 
 struct clist_def srvinf_clist_def = {
@@ -159,13 +166,13 @@ static void clist_column_set_title (GtkCList *clist, struct clist_def *cldef,
   char buf[128];
 
   if (set_mark) {
-    g_snprintf (buf, 128, "%s %c", cldef->cols[clist->sort_column].name, 
+    g_snprintf (buf, 128, "%s %c", _(cldef->cols[clist->sort_column].name), 
                         (clist->sort_type == GTK_SORT_DESCENDING)? '>' : '<');
     gtk_label_set (GTK_LABEL (cldef->cols[clist->sort_column].widget), buf);
   }
   else {
     gtk_label_set (GTK_LABEL (cldef->cols[clist->sort_column].widget), 
-                                        cldef->cols[clist->sort_column].name);
+                                        _(cldef->cols[clist->sort_column].name));
   }
 }
 
@@ -215,7 +222,7 @@ GtkWidget *create_cwidget (GtkWidget *scrollwin, struct clist_def *cldef) {
 
     alignment = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
 
-    label = gtk_label_new (cldef->cols[i].name);
+    label = gtk_label_new (_(cldef->cols[i].name));
     gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
     gtk_container_add (GTK_CONTAINER (alignment), label);
     gtk_widget_show (label);
@@ -273,7 +280,7 @@ void source_ctree_show_node_status (GtkWidget *ctree, struct master *m) {
   gtk_ctree_get_node_info (GTK_CTREE (ctree), node, NULL, NULL,
                                  NULL, NULL, NULL, NULL, &is_leaf, &expanded);
 
-  gtk_ctree_set_node_info (GTK_CTREE (ctree), node, m->name, 4,
+  gtk_ctree_set_node_info (GTK_CTREE (ctree), node, _(m->name), 4,
 			      (pix)? pix->pix : NULL, (pix)? pix->mask : NULL,
 			      (pix)? pix->pix : NULL, (pix)? pix->mask : NULL,
                               is_leaf, expanded);
@@ -424,7 +431,7 @@ static void fill_source_ctree (GtkWidget *ctree) {
 
 
 GtkWidget *create_source_ctree (GtkWidget *scrollwin) {
-  char *titles[1] = { "Source" };
+  char *titles[1] = { _("Source") };
   GtkWidget *ctree;
 
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin), 
@@ -664,7 +671,7 @@ void ui_done (void) {
 }
 
 
-void restore_main_window_geometry () {
+void restore_main_window_geometry (void) {
   int height, width;
   int pane1, pane2, pane3;
 
